#include "pcf8591.h"


void extadc_set_voltage (unsigned char voltage, unsigned char adress)
{
	extadc_init ();
	extadc_send_start ();
	extadc_send_add_rw (adress, 0);
	extadc_send_byte (0b01000000);
	extadc_send_byte (voltage);
	extadc_send_stop ();
}


unsigned char extadc_get_voltage (unsigned char pin, unsigned char adress)
{
	unsigned char ad_value = 0;

	extadc_init ();
	extadc_send_start ();
	extadc_send_add_rw (adress, 1);
	extadc_read_byte ();
	ad_value = extadc_read_last_byte ();
	extadc_send_stop ();

	return ad_value;
}

void extadc_init (void)
{
	/*settings for 8MHz --> calculation: datasheet page 166
	EEPROM is specified up to 1MHz (SCL), I will use 250KHz
	Prescaler Bits TWPS0:1 in TWSR are unnecessary --> default: Prescaler=1
	SCL = CPU/(16+2*TWBR*4)*/
	//TWBR = 0xC6;				// ca. 40kHz(should be 10kHz)
	//TWBR = 0x24;				// ca. 183kHz(should be 100kHz)
	TWBR = 0x48;				// 100 kHz SCL
	//TWBR = 0x97;
}


unsigned char extadc_send_start (void)
{
	/*writing a one to TWINT clears it, TWSTA=Start, TWEN=TWI-enable*/
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
	/*wait, until start condition has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWSR;
}


void extadc_send_stop (void)
{
	/*writing a one to TWINT clears it, TWSTO=Stop, TWEN=TWI-enable*/
	TWCR = (1<<TWINT) | (1<<TWSTO) | (1<<TWEN);	
}


unsigned char extadc_send_add_rw (unsigned char address, unsigned char rw)
{
	/*address can be 0 .. 3; rw=0 --> write, rw=1 --> read*/
	unsigned char addr_byte = 0;
	/*shift address one bit left*/
	addr_byte = address << 1;
	/*set RW-Bit, if necessary*/
	addr_byte |= rw;
	/*0b10100xxx --> address of EEPROM*/
	addr_byte |= 0b10010000;
	/*TWDR contains byte to send*/
	TWDR = addr_byte;
	/*send content of TWDR*/
	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until address has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWSR;
}


unsigned char extadc_send_byte (unsigned char byte)
{
	/*TWDR contains byte to send*/
	TWDR = byte;
	/*send content of TWDR*/
	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until byte has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWSR;
}


unsigned char extadc_read_byte (void)
{
	/*send content of TWDR; TWEA = enable ACK*/
	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN);
	/*wait, until byte has been received --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWDR;
}


unsigned char extadc_read_last_byte (void)
{
	/*send content of TWDR; TWEA = enable ACK*/
	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until byte has been received --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWDR;
}