/*
  usart_lib.h ...
  Funktion: formatierte Ein-/Ausgabe
  Autor: R. Lachmayer
  Datum: 11.2010
*/

#include <avr/io.h>
#include <stdint.h>
#include <stdio.h>

#define BAUD 9600UL
#define F_CPU 16000000UL
#define UBRR_BAUD ((F_CPU/(16UL*BAUD))-1)

void uart_init(void);
static int uart_getchar();
static int uart_putchar(char c, FILE *stream);

/*-----------------------------------------------------------------------------
  void uart_init(void) ...
  Funktion: serielle Schnittstelle UART0 initialisieren
    9600 Baud, 8 Datenbit, 1 Stoppbit, keine Paritt
Freigabe von Sender und Empfnger
  Eingabe: ---
  Rckgabe: ---
-----------------------------------------------------------------------------*/

// USART initialisieren
void uart_init(void)
{
	// Baudrate einstellen (Normaler Modus)
	UBRR0H = (uint8_t) (UBRR_BAUD>>8);
	UBRR0L = (uint8_t) (UBRR_BAUD & 0x0ff);
	// Aktivieren von receiver und transmitter
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	// Einstellen des Datenformats: 8 Datenbits, 1 Stoppbit, keine Paritt
	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);
}

/*-----------------------------------------------------------------------------
  static int uart_putchar(char c, FILE *stream) ...
  Funktion: warten auf Bereitschaft und senden eines Zeichens ber UART0
  Eingabe:
    char c ... zu sendendes Zeichen
FILE *stream ... Ausgabe-Datenstrom
  Rckgabe: int uart_putchar() ... 0
-----------------------------------------------------------------------------*/
static int uart_putchar(char c, FILE *stream)
{
	if (c == '\n')
	uart_putchar('\r', stream);
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = c;
	return 0;
}

/*-----------------------------------------------------------------------------
  unsigned char uart_getchar() ...
  Funktion: warten auf und lesen eines Zeichens von UART0
  Eingabe: ---
  Rckgabe: unsigned char uart_getchar() ... empfangenes Zeichen
-----------------------------------------------------------------------------*/
static int uart_getchar()
{
	// Warten bis Daten empfangen wurden
	while ( !(UCSR0A & (1<<RXC0)) );
	// Empfangsregister auslesen
	return(UDR0);
}